#include <Plasma/FrameSvg>
#include <Plasma/Theme>
#include <Plasma/Wallpaper>
#include <KApplication>
#include <qfile.h>
#include <qtextstream.h>
#include <kstandarddirs.h>

#include <Plasma/Plasma>
#include <QPainter>

#include <QTime>
#include <QImage>
#include <QString>
#include <QPainter>
#include <QPixmap>
#include <QDesktopWidget>

#include <KIcon>

using namespace std;


#include <kcmdlineargs.h>
#include <klocale.h>
#include <kaboutdata.h>

const int iconSize = 24, textHeight = 15;
const int barW = iconSize * 7, barH = iconSize;
const int inputW = 200, inputH = textHeight * 2 + 6;

void saveThemeFile(QString themeFileName,QString imagePath, bool noborder)
{
	QFile f(themeFileName);

	if (f.open(QFile::WriteOnly | QFile::Truncate)) {
		QTextStream out(&f);
		Plasma::FrameSvg svg;
		svg.setImagePath(imagePath);
		svg.setEnabledBorders(Plasma::FrameSvg::AllBorders);
		svg.resizeFrame(QSize(200, 200));

        Plasma::Theme *theme = Plasma::Theme::defaultTheme();
        QColor color = theme->color(Plasma::Theme::TextColor);

		qreal left = 0;
		qreal top = 0;
		qreal right = 0;
		qreal bottom = 0;

		svg.getMargins(left,top,right,bottom);

        int ileft = left, iright = right, itop = top, ibottom = bottom;

        int r,g,b;
        color.getRgb(&r, &g, &b);
        int ar = 255 -r, ag = 255 -g, ab = 255 -b;
        int mleft, mright, mtop, mbottom;
        if (noborder)
        {
            mleft= 0;
            mright = 0;
            mtop = 0;
            mbottom = 0;
        }
        else
        {
            mleft = ileft; mright=iright; mtop = itop; mbottom = ibottom;
        }

		out<<QString(
"[SkinInfo]\n"
"Name=Plasma\n"
"Version=0.1\n"
"Author=CSSlayer\n"
"Desc=Based on plasma\n"
"\n"
"[SkinFont]\n"
"FontSize=15\n"
"TipColor=%5 %6 %7\n"
"InputColor=%5 %6 %7\n"
"IndexColor=%5 %6 %7\n"
"FirstCandColor=%5 %6 %7\n"
"UserPhraseColor=%5 %6 %7\n"
"CodeColor=%5 %6 %7\n"
"OtherColor=%5 %6 %7\n"
"ActiveMenuColor=%8 %9 %10\n"
"InactiveMenuColor=%5 %6 %7\n"
"\n"
"[SkinMainBar]\n"
"BackImg=bar.png\n"
"Logo=logo.png\n"
"Eng=fcitx-eng.png\n"
"Active=fcitx-chn.png\n"
"MarginTop=%17\n"
"MarginBottom=%18\n"
"MarginLeft=%19\n"
"MarginRight=%20\n"
"\n"
"[SkinInputBar]\n"
"BackImg=input.png\n"
"MarginTop=%1\n"
"MarginBottom=%2\n"
"MarginLeft=%3\n"
"MarginRight=%4\n"
"InputPos=%11\n"
"OutputPos=%12\n"
"CursorColor=99 99 99\n"
"BackArrow=next.png\n"
"ForwardArrow=prev.png\n"
"BackArrowX=%13\n"
"BackArrowY=%14\n"
"ForwardArrowX=%15\n"
"ForwardArrowY=%16\n"
"\n"
"[SkinTrayIcon]\n"
"Active=active.png\n"
"Inactive=inactive.png\n"
"\n"
"[SkinMenu]\n"
"BackImg=menu.png\n"
"MarginTop=%1\n"
"MarginBottom=%2\n"
"MarginLeft=%3\n"
"MarginRight=%4\n"
"ActiveColor=%5 %6 %7\n"
"LineColor=%5 %6 %7\n"
).arg(itop).arg(ibottom).arg(ileft).arg(iright).arg(r).arg(g).arg(b).arg(ar).arg(ag).arg(ab).arg(textHeight + 2).arg(textHeight * 2 + 6).arg(right + 16).arg(top).arg(right + 32).arg(top).arg(mtop).arg(mbottom).arg(mleft).arg(mright);
		f.close();
	}

}


void saveThemeElement(QString filename, QPixmap svg, int a)
{
    int w = svg.width();
    int h = svg.height();

	QPixmap pixmap(w,h);
	pixmap.fill(Qt::transparent);
	QPainter p(&pixmap);
	p.drawPixmap(pixmap.rect(),svg,QRectF(0,0,w,h));
	p.end();

	if(a >= 0 && a <= 255) {
		QPixmap alpha(w,h);
		QPainter p1(&alpha);
		p1.fillRect(QRectF(0,0,w,h),QColor(a,a,a));
		p1.end();
		pixmap.setAlphaChannel(alpha);
	}

	pixmap.save(filename,"PNG");
}


void saveBoxPixmaps(QString themePath,QString imagePath, int alpha, bool noborder)
{
    QSize s(200,200);
	Plasma::FrameSvg svg;
	svg.setImagePath(imagePath);
	svg.setEnabledBorders(Plasma::FrameSvg::AllBorders);
	svg.resizeFrame(s);

	qreal left = 0;
	qreal top = 0;
	qreal right = 0;
	qreal bottom = 0;

	svg.getMargins(left,top,right,bottom);

    QSize barSize(left + right + barW, top + bottom + barH);
    QSize inputSize(left + right + inputW, top + bottom + inputH);
    QSize menuSize(left + right + 130, top + bottom + 200);

        svg.resizeFrame(barSize);
    if (noborder)
    {
    	saveThemeElement(themePath+"bar.png",svg.framePixmap(), 0);
    }
    else
    {
    	saveThemeElement(themePath+"bar.png",svg.framePixmap(),alpha);
    }

    svg.resizeFrame(inputSize);
	saveThemeElement(themePath+"input.png",svg.framePixmap(),alpha);

    svg.resizeFrame(menuSize);
	saveThemeElement(themePath+"menu.png",svg.framePixmap(),alpha);
}


static KCmdLineOptions options ()
{
	KCmdLineOptions options;
	options.add("a");
	options.add("alpha <alpha>",	ki18n("Set alpha of border (0-255)"));
	options.add("p");
	options.add("imagepath <imagepath>",		ki18n("Set image path of Plasma svg. "
				"'dialogs/background' and 'opaque/dialogs/backgrounds' are recommended. "
				"default is 'dialogs/backgrounds'."));
    options.add("noborder", ki18n("No order for mainbar, good for place on the panel"));
	return options;
}


int main( int argc, char **argv )
{

	KAboutData aboutData( "FcitxThemeGenerator", 
			"FcitxThemeGenerator", 
			ki18n("FcitxThemeGenerator"), 
			"0.3", 
			ki18n("A theme generator for Fcitx"),
			KAboutData::License_GPL, ki18n("Copyright (c) 2009 Weng Xuetian") );

	KCmdLineArgs::init( argc, argv, &aboutData );
	KCmdLineArgs::addCmdLineOptions(options());

	KApplication a;
    bool noborder = false;

	QString myFcitxHome= QString(getenv("HOME"))+"/.config/fcitx/skin/plasma/";

	if (!QFile(myFcitxHome).exists())  KStandardDirs::makeDir(myFcitxHome,0755);

	QString imagePath="dialogs/background";

	int alpha = -1;

	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if (args->isSet("alpha")) alpha = args->getOption("alpha").toInt();
	if (args->isSet("imagepath")) imagePath = args->getOption("imagepath");
    noborder = !args->isSet("border");

	args->clear();

	saveBoxPixmaps(myFcitxHome, imagePath, alpha, noborder);
	saveThemeFile(myFcitxHome + "fcitx_skin.conf", imagePath, noborder);

    QString cmd = "tar xvf png.tar.gz -C " + myFcitxHome;
    system(cmd.toStdString().data());

	return 0;
}

